@@ -1,9 +1,10 @@
-/*
+/* -*- mode: c; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
+
   digest -- hash digest functions for R
 
-  Copyright (C) 2003 - 2009  Dirk Eddelbuettel <edd@debian.org>
+  Copyright (C) 2003 - 2011  Dirk Eddelbuettel <edd@debian.org>
 
-  $Id: digest.c 24 2009-09-24 12:36:55Z edd $
+  $Id: digest.c 42 2011-09-14 17:11:45Z edd $
 
   This file is part of the digest packages for GNU R.
   It is made available under the terms of the GNU General Public
@@ -28,24 +29,26 @@
 #include <R.h>
 #include <Rdefines.h>
 #include <Rinternals.h>
-#include "sha1.h"		
-#include "sha256.h"		
+#include "sha1.h"   
+#include "sha256.h"   
 #include "md5.h"
 #include "zlib.h"
 
 unsigned long ZEXPORT digest_crc32(unsigned long crc,
-				   const unsigned char FAR *buf,
-				   unsigned len);
+                                   const unsigned char FAR *buf,
+                                   unsigned len);
 
-SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
+SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip, SEXP Leave_raw) {
   FILE *fp=0;
   char *txt;
   int algo = INTEGER_VALUE(Algo);
   int  length = INTEGER_VALUE(Length);
   int skip = INTEGER_VALUE(Skip);
+  int leaveRaw = INTEGER_VALUE(Leave_raw);
   SEXP result = NULL;
-  char output[65];		/* 33 for md5, 41 for sha1, 65 for sha256 */
+  char output[65];    /* 33 for md5, 41 for sha1, 65 for sha256 */
   int nChar;
+  int output_length = -1;
   if (IS_RAW(Txt)) { /* Txt is either RAW */
     txt = (char*) RAW(Txt);
     nChar = LENGTH(Txt);
@@ -63,35 +66,40 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
   if (length>=0 && length<nChar) nChar = length;
   
   switch (algo) {
-    case 1: {			/* md5 case */
+    case 1: {     /* md5 case */
       md5_context ctx;
+      output_length = 16;
       unsigned char md5sum[16];
       int j;
-
       md5_starts( &ctx );
       md5_update( &ctx, (uint8 *) txt, nChar);
       md5_finish( &ctx, md5sum );
+      memcpy(output, md5sum, 16);
 
-      for(j = 0; j < 16; j++) {
-		sprintf(output + j * 2, "%02x", md5sum[j]);
-      }
+      if (!leaveRaw)
+        for(j = 0; j < 16; j++) 
+          sprintf(output + j * 2, "%02x", md5sum[j]);
+        
       break;
     }
-    case 2: {			/* sha1 case */
+    case 2: {     /* sha1 case */
       int j;
       sha1_context ctx;
+      output_length = 20;
       unsigned char sha1sum[20];
 
       sha1_starts( &ctx );
       sha1_update( &ctx, (uint8 *) txt, nChar);
       sha1_finish( &ctx, sha1sum );
+      memcpy(output, sha1sum, 20);
 
-      for( j = 0; j < 20; j++ ) {
-		sprintf( output + j * 2, "%02x", sha1sum[j] );
-      }
+      if (!leaveRaw)
+        for( j = 0; j < 20; j++ ) 
+          sprintf( output + j * 2, "%02x", sha1sum[j] );
+        
       break;
     }
-    case 3: {			/* crc32 case */
+    case 3: {     /* crc32 case */
       unsigned long val, l;
       l = nChar;
 
@@ -102,23 +110,27 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
 
       break;
     }
-    case 4: {			/* sha256 case */
+    case 4: {     /* sha256 case */
       int j;
       sha256_context ctx;
+      output_length = 32;
       unsigned char sha256sum[32];
 
       sha256_starts( &ctx );
       sha256_update( &ctx, (uint8 *) txt, nChar);
       sha256_finish( &ctx, sha256sum );
 
-      for( j = 0; j < 32; j++ ) {
-		sprintf( output + j * 2, "%02x", sha256sum[j] );
-      }
+      memcpy(output, sha256sum, 32);
+      if(!leaveRaw)
+        for( j = 0; j < 32; j++ ) 
+          sprintf( output + j * 2, "%02x", sha256sum[j] );
+        
       break;
     }
-    case 101: {			/* md5 file case */
+    case 101: {     /* md5 file case */
       int j;
       md5_context ctx;
+      output_length = 16;
       unsigned char buf[1024];
       unsigned char md5sum[16];
 
@@ -130,7 +142,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       md5_starts( &ctx );
       if (length>=0) {  
         while( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0 
-	       && length>0) {
+               && length>0) {
           if (nChar>length) nChar=length;
           md5_update( &ctx, buf, nChar );
           length -= nChar;
@@ -141,12 +153,16 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       }
       fclose(fp);
       md5_finish( &ctx, md5sum );
-      for(j = 0; j < 16; j++) sprintf(output + j * 2, "%02x", md5sum[j]);
+      memcpy(output, md5sum, 16);
+      if (!leaveRaw)
+        for(j = 0; j < 16; j++) 
+          sprintf(output + j * 2, "%02x", md5sum[j]);
       break;
     }
-    case 102: {			/* sha1 file case */
+    case 102: {     /* sha1 file case */
       int j;
       sha1_context ctx;
+      output_length = 20;
       unsigned char buf[1024];
       unsigned char sha1sum[20];
       
@@ -158,7 +174,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       sha1_starts ( &ctx );
       if (length>=0) {  
         while( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0 
-	       && length>0) {
+               && length>0) {
           if (nChar>length) nChar=length;
           sha1_update( &ctx, buf, nChar );
           length -= nChar;
@@ -169,10 +185,13 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       }
       fclose(fp);
       sha1_finish ( &ctx, sha1sum );
-      for( j = 0; j < 20; j++ ) sprintf( output + j * 2, "%02x", sha1sum[j] );
+      memcpy(output, sha1sum, 20);
+      if(!leaveRaw)
+        for( j = 0; j < 20; j++ ) 
+          sprintf( output + j * 2, "%02x", sha1sum[j] );
       break;
     }
-    case 103: {			/* crc32 file case */
+    case 103: {     /* crc32 file case */
       unsigned char buf[1024];
       unsigned long val;
       
@@ -184,7 +203,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       val  = digest_crc32(0L, 0, 0);
       if (length>=0) {  
         while( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0 
-	       && length>0) {
+               && length>0) {
           if (nChar>length) nChar=length;
           val  = digest_crc32(val , buf, (unsigned) nChar);
           length -= nChar;
@@ -197,9 +216,10 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       sprintf(output, "%2.2x", (unsigned int) val);
       break;
     }
-    case 104: {			/* sha256 file case */
+    case 104: {     /* sha256 file case */
       int j;
       sha256_context ctx;
+      output_length = 32;
       unsigned char buf[1024];
       unsigned char sha256sum[32];
       
@@ -211,7 +231,7 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       sha256_starts ( &ctx );
       if (length>=0) {  
         while( ( nChar = fread( buf, 1, sizeof( buf ), fp ) ) > 0 
-	       && length>0) {
+               && length>0) {
           if (nChar>length) nChar=length;
           sha256_update( &ctx, buf, nChar );
           length -= nChar;
@@ -222,8 +242,10 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
       }
       fclose(fp);
       sha256_finish ( &ctx, sha256sum );
-      for( j = 0; j < 32; j++ ) 
-		  sprintf( output + j * 2, "%02x", sha256sum[j] );
+      memcpy(output, sha256sum, 32);
+      if(!leaveRaw)
+        for( j = 0; j < 32; j++ ) 
+          sprintf( output + j * 2, "%02x", sha256sum[j] );
       break;
     }
 
@@ -233,8 +255,13 @@ SEXP digest(SEXP Txt, SEXP Algo, SEXP Length, SEXP Skip) {
     }  
   }
 
-  PROTECT(result=allocVector(STRSXP, 1));
-  SET_STRING_ELT(result, 0, mkChar(output));
+  if (leaveRaw && output_length > 0) {
+    PROTECT(result=allocVector(RAWSXP, output_length));
+    memcpy(RAW(result), output, output_length);
+  } else {
+    PROTECT(result=allocVector(STRSXP, 1));
+    SET_STRING_ELT(result, 0, mkChar(output));
+  }
   UNPROTECT(1);
 
   return result;
